#include <bits/stdc++.h>
using namespace std;

const int INF = 1e9 + 10;

struct edge {
  int to, capacity, weight, flow;
};

const int N = 55;
const int M = N * N * 2;

edge arr[M];
vector<int> g[N];
int cnt;

void addEdge(int from, int to, int capacity, int weight) {
  arr[cnt] = {to, capacity, weight, 0};
  g[from].push_back(cnt++);
}

int potential[N];

void initPotentials(int vertexCount, int S) {
  vector<int> dist(vertexCount, INF);
  dist[S] = 0;
  for (int i = 0; i < vertexCount - 1; i++) {
    for (int v = 0; v < vertexCount; v++) {
      for (int id : g[v]) {
        if (dist[v] + arr[id].weight < dist[arr[id].to] && arr[id].capacity > arr[id].flow) {
          dist[arr[id].to] = dist[v] + arr[id].weight;
        }
      }
    }
  }
  for (int i = 0; i < vertexCount; i++) {
    potential[i] = dist[i];
  }
}

pair<int, int> dijkstra(int vertexCount, int S, int T) {
  vector<int> dist(vertexCount, INF);
  vector<pair<int, int>> from(vertexCount, make_pair(-1, -1));
  vector<bool> used(vertexCount, false);
  dist[S] = 0;
  for (int iteration = 0; iteration < vertexCount; iteration++) {
    pair<int, int> best = {INF, INF};
    for (int j = 0; j < vertexCount; j++) {
      if (!used[j]) {
        best = min(best, {dist[j], j});
      }
    }
    int v = best.second;
    used[v] = true;
    for (auto it : g[v]) {
      int u = arr[it].to;
      int cost = arr[it].weight + potential[v] - potential[u];
      if (arr[it].flow < arr[it].capacity && dist[v] + cost < dist[u]) {
        dist[u] = dist[v] + cost;
        from[u] = {v, it};
      }
    }
  }
  if (dist[T] == INF) {
    return {INF, 0};
  }
  int now = T, mn = INF;
  while (from[now].first != -1) {
    mn = min(mn, arr[from[now].second].capacity - arr[from[now].second].flow);
    now = from[now].first;
  }
  now = T;
  while (from[now].first != -1) {
    arr[from[now].second].flow += mn;
    arr[from[now].second ^ 1].flow -= mn;
    now = from[now].first;
  }
  int result = (dist[T] - potential[S] + potential[T]) * mn;
  for (int i = 0; i < vertexCount; i++) {
    if (dist[i] < INF) {
      potential[i] += dist[i];
    }
  }
  return {result, mn};  
}

int main() {
  ios_base::sync_with_stdio(0);
  cin.tie(0); 
  int n;
  cin >> n;
  int sum = 0;
  vector<int> a(n);
  for (int i = 0; i < n; i++) {
    cin >> a[i];
    sum += a[i];
  }  
  if (sum & 1) {
    cout << -1 << '\n';
    return 0;
  }
  vector<vector<int>> cost(n, vector<int>(n));
  for (int i = 0; i < n; i++) {
    for (int j = 0; j < n; j++) {
      cin >> cost[i][j];
    }
  }
  int vertexCount = 2 * n + 2;
  //S, T, two vertices for each index
  int S = 2 * n, T = 2 * n + 1;
  int answer = INF;
  for (int maskLeft = 0; maskLeft < (1 << n); maskLeft++) {
    cnt = 0;
    for (int i = 0; i < vertexCount; i++) {
      g[i] = {};
    }

    auto getBit = [&](int x, int bit) {
      return (x >> bit) & 1;
    };

    for (int i = 0; i < n; i++) {
      int leftCapacity = (a[i] + getBit(maskLeft, i)) / 2, 
      rightCapacity = a[i] - leftCapacity;
      addEdge(S, i, leftCapacity, 0);
      addEdge(i, S, 0, 0);

      addEdge(n + i, T, rightCapacity, 0);
      addEdge(T, n + i, 0, 0);
      for (int j = n; j < 2 * n; j++) {
        addEdge(i, j, INF, cost[i][j - n]);
        addEdge(j, i, 0, -cost[i][j - n]);
      }
    } 
    initPotentials(vertexCount, S);
    int sumOfCosts = 0, currentFlowSize = 0;
    while (currentFlowSize <= sum / 2) {
      auto result = dijkstra(vertexCount, S, T);
      if (result.first >= INF) {
        if (currentFlowSize == sum / 2) {
          answer = min(answer, sumOfCosts);
        }
        break;
      }
      currentFlowSize += result.second;
      sumOfCosts += result.first;
    } 
  }
  assert(answer < INF);
  cout << answer << '\n';
  return 0;
}     